﻿using System.Reflection;
using System.Text.Encodings.Web;
using System.Text.Json;
using System.Text.Json.Serialization.Metadata;
using System.Text.Unicode;
using System.Xml;

namespace CoroutineSerializationWithTextJson {

	class Program {
		static IEnumerator<int> NumberList() {
			int counter = 0;
			for (int i = 0; i < 10; i++) {
				int sum = counter + 1000;
				counter += 2;
				if (i % 2 == 0) {
					int subValue = sum - 1;
					yield return i * 10 + subValue;
				} else {
					yield return i * 100 + sum;
				}
			}
		}

		static void Main(string[] args) {
			var optionsWithPrivateFields = new JsonSerializerOptions {
				WriteIndented = true,
				TypeInfoResolver = new DefaultJsonTypeInfoResolver {
					Modifiers = { AddPrivateFieldsModifier }
				}
			};

			//

			var e = NumberList();

			Console.WriteLine();
			Console.WriteLine("Enumerating 5 items:");
			for (int i = 0; i < 5; i++) {
				e.MoveNext();
				Console.WriteLine(e.Current);
			}

			Console.WriteLine();
			Console.WriteLine("Enumerator state machine fields via Reflection:");
			var fields = e.GetType().GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
			foreach (var f in fields) {
				Console.WriteLine($"{f.Name} == {f.GetValue(e)}");
			}

			//

			var optionsWithPrivateFieldsRelaxedEscaping = new JsonSerializerOptions {
				WriteIndented = true,
				TypeInfoResolver = new DefaultJsonTypeInfoResolver {
					Modifiers = { AddPrivateFieldsModifier }
				},
				// Encoder = JavaScriptEncoder.Create(UnicodeRanges.All)
				Encoder = JavaScriptEncoder.UnsafeRelaxedJsonEscaping
			};

			var eJson = JsonSerializer.Serialize((object) e, optionsWithPrivateFieldsRelaxedEscaping);

			Console.WriteLine();
			Console.WriteLine("Enumerator state machine in JSON:");
			Console.WriteLine(eJson);
		}

		static void AddPrivateFieldsModifier(JsonTypeInfo jsonTypeInfo) {
			if (jsonTypeInfo.Kind != JsonTypeInfoKind.Object)
				return;

			var fields = jsonTypeInfo.Type.GetFields(BindingFlags.Instance | BindingFlags.NonPublic);
			foreach (FieldInfo field in fields) {
				JsonPropertyInfo jsonPropertyInfo = jsonTypeInfo.CreateJsonPropertyInfo(field.FieldType, field.Name);
				jsonPropertyInfo.Get = field.GetValue;
				jsonPropertyInfo.Set = field.SetValue;

				jsonTypeInfo.Properties.Add(jsonPropertyInfo);
			}
		}
	}
}